package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.IOException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.io.FileDescriptor;
import java.net.InetSocketAddress;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("sun.nio.ch.UnixAsynchronousServerSocketChannelImpl")
public class UnixAsynchronousServerSocketChannelImplRMHooks
{
    private static final UnixAsynchronousServerSocketChannelImplRMHooks.NativeDispatcher nd;
    protected volatile InetSocketAddress localAddress;
    
    public UnixAsynchronousServerSocketChannelImplRMHooks() {
        this.localAddress = null;
    }
    
    @InstrumentationMethod
    private int accept(FileDescriptor fileDescriptor, FileDescriptor fileDescriptor2, InetSocketAddress[] array) throws IOException {
        int accept = this.accept(fileDescriptor, fileDescriptor2, array);
        ResourceIdImpl of = ResourceIdImpl.of(fileDescriptor2);
        if (of != null) {
            ResourceRequest approver = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(fileDescriptor2);
            long request = 0L;
            long n = 0L;
            try {
                try {
                    request = approver.request(1L, of);
                    if (request < 1L) {
                        throw new IOException("Resource limited: too many open file descriptors");
                    }
                }
                catch (ResourceRequestDeniedException cause) {
                    throw new IOException("Resource limited: too many open file descriptors", cause);
                }
                n = 1L;
            }
            finally {
                if (n == 0L) {
                    try {
                        UnixAsynchronousServerSocketChannelImplRMHooks.nd.close(fileDescriptor2);
                    }
                    catch (IOException ex) {}
                }
                else {
                    approver.request(-(request - 1L), of);
                }
            }
        }
        return accept;
    }
    
    @InstrumentationMethod
    void implClose() throws IOException {
        try {
            this.implClose();
        }
        finally {
            if (this.localAddress != null) {
                ApproverGroup.SOCKET_OPEN_GROUP.getApprover(this).request(-1L, ResourceIdImpl.of(this.localAddress));
            }
        }
    }
    
    static {
        nd = null;
    }
    
    abstract class NativeDispatcher
    {
        abstract void close(FileDescriptor p0) throws IOException;
    }
}
